#include <stdio.h>
#include "cpudef.h"
#include "ram.h"
#include "opcodes.h"

/* predefines */
int reset_68k(void);
int M68000_step_dyn(void);
int M68000_step_sta(void);
int locate_opcode(unsigned short int p);
int M68000_compile(void);
int messager(int packet);
int bumb(void);
int (*M68000_step)(void) = M68000_step_dyn;
int getopname(char *opname, unsigned long address);
int handle_int(int type);

/* global varibles */
genst gen;

t_cpu m68000 = {
     reset_68k,
     bumb,
     M68000_step_dyn,
     handle_int,
     messager,
     getopname,
     fakebus,
     &gen,
};

t_pipeline pipeline;
t_op core[] = {
   /* Mask, Op, cycles, function */
   { 0xffff, 0x4e71, 1, _nop, "nop"     },
   { 0xff00, 0x0200, 3, _andi, "andi"   },
   { 0xff00, 0x4a00, 3, _tst, "tst"     },
   { 0xfb80, 0x4880, 3, _movem, "movem" },
   { 0xf1c0, 0x41c0, 3, _lea, "lea"     },
   { 0xf000, 0x6000, 2, _bcc, "bcc"     },
   { 0xf000, 0x1000, 3, _move, "move"   },
   { 0xf000, 0x2000, 3, _move, "move"   },
   { 0xf000, 0x3000, 3, _move, "move"   },   
   { 0, 0, 0, NULL}
};
t_opc *ccore;



int M68000_step_dyn(void)
{
    int d=0;
    register id;
    pipeline.decode=read_word(gen.pc);
    while(core[d].mask)
    {
       if(( core[d].mask & pipeline.decode ) == core[d].op){
           opdsp=2;
           id=core[d].c();
           if(id) 
           {  /* what to do if the opcode worked */
              gen.pc+=opdsp;
              gen.cycles+=core[d].cycles;
           }
           return id;
       }
       else d++;
    }
    return 0;
}
int M68000_step_sta(void)
{
     register id;
     opdsp=2;
     if((id = ccore[pipeline.decode=read_word(gen.pc)].c())) 
     {  /* what to do if the opcode worked */
        gen.pc+=opdsp;
        gen.cycles+=ccore[pipeline.decode].cycles;
     }
     return id;
}
int locate_opcode(unsigned short int p)
{
    int d=0;
    while(core[d].mask)
    {
       if(( core[d].mask & p ) == core[d].op){
           return d;
       }
       else d++;
    }
    return -1;

}
int bumb(void)
{
    return 0;

}
int M68000_compile(void)
{
    int p,v;
    printf("Allocating cpu space...");
    if( (ccore = (t_opc *) malloc(sizeof(t_opc)*65536)) == NULL)
    {
             printf("Failed.\n");
             return 1;
    }
    printf("Done.\n");
    printf("Compiling m68000 cpu...");
    for(p=0;p<65536;p++)
    {
         if( (v=locate_opcode(p)) == -1)
         {
                ccore[p].cycles=0x1234;
                ccore[p].c=bumb;
         }
         else
         {
                ccore[p].cycles=core[v].cycles;
                ccore[p].c=core[v].c;
         }
    }
    m68000.step = M68000_step_sta;
    printf("Done.\n");
    return 0;
}

int messager(int packet)
{
    switch(packet)
    {
      case COMPILE_CORE:
      {
         return M68000_compile();
      }
      default:
         return 0;
    }
}

int reset_68k(void)
{
   int p;
   for(p=0;p<8;p++)
   {
      gen.a[p]=0;
      gen.d[p]=0;
      gen.z=0;
      gen.c=0;
      gen.n=0;
      gen.x=0;
      gen.v=0;
   
   }
   gen.a[7]=read_long(0);
   gen.pc=read_long(4);
   gen.cycles=0;
   return 1;
}
int getopname(char *opname, unsigned long address)
{
   int m;
   m=locate_opcode(read_word(address));
   if(m == -1)
   {
     strcpy(opname,"");
     return 0;
   }
   strcpy(opname,core[m].rawname);
}
int handle_int(int type)
{
   return type;

}
        
        
