#include "cpudef.h"
#include "core.h"


extern genst gen;
extern t_pipeline pipeline;
extern t_cpu m68000;
int opdsp;
int nopdsp;
unsigned char busstat;
unsigned char etype;
unsigned long int eaddy;
unsigned char siz[] = { 1,2,4 };

/* predefs */

int sevens(char reg,char size,char pack);
unsigned int fakebus(unsigned char d,unsigned char size,unsigned long int address,unsigned long int data);

/* normal functions */

int
get_ea(char mode,char reg,char size,char pack)
{
     nopdsp=0;
     printf("get_EA [mode %01x,reg %01x]:%01x\n",mode,reg,size);
         
     switch(mode)
     {
        case 0:/*Data Register Direct*/
        {         
           etype=2;
           eaddy=reg;
           return gen.d[reg];
        }
        case 1:/*Address Register Direct*/
        {
           etype=1;
           eaddy=reg;
           return gen.a[reg];
        }
        case 2:/*Address Register Indirect*/
        {
           etype=0;
           eaddy=gen.a[reg];
           return m68000.bus(0,size,eaddy,0);
        }
        case 3:/*Address Register Indirect with Postincrement*/
        {
           etype=0;
           eaddy=gen.a[reg];
           if((reg==7) && (size==0))
              gen.a[reg] += 2;
           else
              gen.a[reg] += siz[size];
           
           gen.cycles+=2;
           return m68000.bus(0,size,eaddy,0);
        }
        case 4:/*Adress register Indirect with Predecrement*/
        {
           etype=0;
           if((reg==7) && (size==0))
             eaddy=gen.a[reg] - 2;
           else                                   
             eaddy=gen.a[reg] - siz[size];
             
           gen.a[reg]=eaddy;
           gen.cycles+=2;
           return m68000.bus(0,size,eaddy,0);
        }
        case 5:/*Address register Indirect with displacement*/
        {
           etype=0;
           eaddy=read_long(gen.a[reg])+(signed short int) read_word(gen.pc+opdsp);
           opdsp+=nopdsp=2;
           gen.cycles+=2;
           return m68000.bus(0,size,eaddy,0);
        }
        case 7:/*special*/
            return sevens(reg,size,pack);
        default:
        {
           printf("Unimplemented Addressing mode, %01x:%01x\n",mode,reg);
           etype=0;
           eaddy=0;
           return 0;
        }
     }
}

/* etype=2/1/0 reg D/reg A/memory
 * eaddy=the memory address used.
 */

int
siz0(char size)
{
   switch(size)
   {
     case 0:
       return 1;
     case 1:
       return 2;
     case 2:
       return 4;
   }
}

sevens(char reg,char size,char pack)
{
   switch(reg)
   {
      case 1:
      {
         eaddy=m68000.bus(0,2,gen.pc+opdsp,0);
         etype=0;
         opdsp+=nopdsp=4;
         return m68000.bus(0,size,eaddy,0);
      }
      case 2:
      {
         eaddy= (gen.pc+opdsp)+(signed short int) read_word(gen.pc+opdsp);
         opdsp+=nopdsp=2;
         etype=0;
         return m68000.bus(0,size,eaddy,0);
      }
      case 4: /* immediate */
      {
         eaddy= gen.pc+opdsp;
         if(size >= 2) opdsp+=nopdsp=4;
         else opdsp+=nopdsp=2;
         etype=0;
         return m68000.bus(0,size,eaddy,0);
      }
      default:
      {
         printf("Unimplemented Addressing mode, 7:%01x\n",reg);
         etype=0;
         eaddy=0;
         return 0;
      }     
   
   }
}
put_ea(char mode,char reg,char size,unsigned long value,char pack)
{
     unsigned long int teaddy;
     unsigned char tetype;
     
     printf("put_EA [mode %01x,reg %01x]:%01x,%08x :",mode,reg,size,value);
    
     teaddy=eaddy;
     tetype=etype;
     
     get_ea(mode,reg,size,pack);
     switch(etype)
     {
         case 0:/* memory */
         {
           m68000.bus(1,size,eaddy,value);
           break;
         }
         case 1:/* reg A */
         {
           gen.a[reg]=value;
           break;
         }
         case 2:/* reg d */
         {
           gen.d[reg]=value;
           break;
         }
     
     } 
     eaddy=teaddy;
     etype=teaddy;
}
unsigned int fakebus(unsigned char d,unsigned char size,unsigned long int address,unsigned long int data)
{
   return 0;
}