#include <string.h>
#include <zlib.h>
#include "main.h"
#include "m68000.h"

extern unsigned char *gen_ram;
extern switches genux;

FILE *cart;
gzFile in;
int filesize;
unsigned char *tmprom;


int load_cart(char *filename)
{
    char carti[266];
    char game[0x31];
    char calcs[1024+256];    
    char carto[266];
    char *ext;
    char *pars;
    char agz=0;
    char rtype;
    unsigned long uncomprLen = 0xffffff;
    int ref;
    
    strncpy(calcs,filename,1024+256);
     
    rtype=0; /* .bin */
    ext = strrchr(filename, '.');
    pars = strrchr(filename, '/');
    if(pars == NULL) pars = filename;
    else if(strlen(pars) > 1) pars++;
    if(strlen(pars) == 1)
    {
      switch(pars[0])
      {
            case '.':
            case '/':
            case '\\':
            case ':':
            {
               printf("Bah! I don't like that filename\n");
               return 1;
            }
      }
    }  
    
    strcpy(carto,"./roms/");
    strncat(carto,pars,265);
    if(strcasecmp(ext, ".gz") == 0)
    {
        agz=1;
        ext[0]='\0';
        for(;((filename <= ext) && ( ext[0] != '.' )); ext--);
    }
    if(strcasecmp(ext, ".smd") == 0) rtype=1;
    
    
    printf("  Loading Cart....:%s\n",pars);
    if(agz)
    {
       if(zlibVersion()[0] != ZLIB_VERSION[0]) 
       {
             printf("Incompatable zlib version.\n");
             wedie(10);
       }
       if(( in = gzopen(calcs, "rb")) == Z_NULL)
       {
            if(( in = gzopen(carto, "rb")) == Z_NULL)       
            {
               printf("File Not found\n");
               return 1;            
            }
       }
       if( (tmprom = (char *) malloc(0xffffff)) == NULL){
           printf("out of memory error\n");
           wedie(8);
       }
       
       uncomprLen = gzread(in, tmprom, (unsigned)uncomprLen);
       gzclose(in);
       filesize=uncomprLen;       
    }
    else
    {
       if(( cart = fopen(filename,"rb")) == NULL)
       {
            strcpy(carti,"./roms/");
            strncat(carti,pars,265);
            if(( cart = fopen(carti,"rb")) == NULL)
            {
                printf("File Not found\n");
                return 1;
            }
         
       }
    
       fseek(cart, 0, SEEK_END);
       if( (filesize = ftell(cart)) > 0xffffff)
              return BAD_CART;

       if( (tmprom = (char *) malloc(0xffffff)) == NULL){
              printf("out of memory error\n");
              wedie(8);
       }
     
       fseek(cart, 0, SEEK_SET);    
       fread(tmprom, filesize, 1, cart);
       fclose(cart);
    }

    switch(rtype)
    {    
       case 1:
       {
          cart_smd();
          break;
       }
       case 0:
       {
          cart_bin();
          break;
       }
    }     
    
    free(tmprom);

     /* state rom info here*/
    memset(game,0,0x31);
    memcpy(game, &gen_ram[0x120], 0x30);
    printf("  [%s] %i Mbits\n",game,(filesize*8)/1048576);
    if(genux.t /* if they did -t in the future */)
    {
         printf("Dumping debuging rom 'genux.bin'\n");
         if(( cart = fopen("genux.bin","wb")) == NULL)
             printf("debuging problem, not dumping\n");
         else
         {
             fwrite(gen_ram, filesize, 1, cart);
             fclose(cart);
         }    
    }
    return 0;
}


int cart_bin()
{
    memcpy(gen_ram,tmprom,filesize);      
    return 0; /* success!!! */  
}

int cart_smd()
{
    register int counter,x,ps;
    ps=0;
    filesize-= 512;
    for(counter = 0; counter < (filesize/8192); counter +=2)
    {
       for(x = 0; x < 8192; x++)
       {
          gen_ram[ps++] = tmprom[ ((counter + 1) * 8192) + x +512];
          gen_ram[ps++] = tmprom[ (counter * 8192) + x + 512];
       }
    }
    return 0;
}

